@@ -1,6 +1,7 @@
-__all__ = ['tracks','fits']
+__all__ = ['tracks','fits','lensing','errors']
 
 from .tracks import *
 from .fits import *
 from .lensing import *
+from .errors import *
 #name = 'astromet'
