@@ -2,6 +2,7 @@ import numpy as np
 import astropy.coordinates
 from astropy import units as u
 from astropy.time import Time
+T = (1.0*u.year).to(u.day).value
 
 from .tracks import *
 
@@ -139,7 +140,7 @@ def mock_obs(ts, phis, racs, decs, err=0, nmeasure=9):
     (i.e. 1D positions along some scan angle, optionlly with errors added)
     Args:
         - ts,       ndarray - Observation times, jyear.
-        - phis,     ndarray - Scanning angles (0 north, 90 east), degrees.
+        - phis,     ndarray - Scanning angles (0 north, pi/2 east), radians.
         - racs,     ndarray - RAcosDec at each scan, mas
         - decs,     ndarray - Dec at each scan, mas
         - err,      float or ndarray - optional normal distributed error to be added (default 0)
@@ -151,9 +152,9 @@ def mock_obs(ts, phis, racs, decs, err=0, nmeasure=9):
     ts= np.repeat(ts, nmeasure)
     phis= np.repeat(phis, nmeasure)
     errs=err*np.random.randn(ts.size)
-    racs= np.repeat(racs, nmeasure) + errs*np.sin(np.deg2rad(phis))
-    decs= np.repeat(decs, nmeasure) + errs*np.cos(np.deg2rad(phis))
-    xs=racs*np.sin(np.deg2rad(phis)) + decs*np.cos(np.deg2rad(phis))
+    racs= np.repeat(racs, nmeasure) + errs*np.sin(phis)
+    decs= np.repeat(decs, nmeasure) + errs*np.cos(phis)
+    xs=racs*np.sin(phis) + decs*np.cos(phis)
     return ts,xs,phis,racs,decs
 
 
@@ -164,7 +165,7 @@ def fit(ts, xs, phis, xerr, ra, dec, G=12, epoch=2016.0):
     Args:
         - ts,          ndarray - source observation times, jyear
         - xs,          ndarray - source 1d positions relative to ra,dec , mas
-        - phis,        ndarray - source observation scan angles, deg
+        - phis,        ndarray - source observation scan angles, radians
         - errs,        ndarray - scan measurement error, mas
         - ra,          float - RA for design_1d, deg
         - dec,         float - Dec for design_1d, deg
@@ -231,6 +232,7 @@ def gaia_results(results):
         for j in range(i):
             gresults[gcoords[j]+'_'+gcoords[i]+'_corr']=\
                 results[coords[j]+'_'+coords[i]+'_corr']
+    # gaia doesn't use a reference ra and dec + offset, just recrods ra and dec in degrees
     gresults['ra']=results['ra_ref']+results['drac']*mas/np.cos(results['dec_ref'])
     gresults['dec']=results['dec_ref']+results['ddec']*mas
 
@@ -297,7 +299,7 @@ def agis(r5d, t, phi, x_err, extra=None, epoch=2016.0, G=None):
     Args:
         - r5d,        ndarray - 5D astrometry of source - (ra, dec (deg), parallax (mas), mura, mudec (mas/y))
         - t,          ndarray - source observation times - (julian days relative to Gaia observation start?)
-        - phi,        ndarray - source observation scan angles
+        - phi,        ndarray - source observation scan angles, radians
         - x_err,      ndarray - scan measurement error
         - extra,  function or None - Takes times and returns offset of centroid from CoM in mas.
     Returns:
@@ -333,7 +335,7 @@ def agis(r5d, t, phi, x_err, extra=None, epoch=2016.0, G=None):
     x_obs  = np.matmul(design, r5d)
     # Excess motion
     if extra is not None:
-        x_obs += np.sum(np.vstack((np.sin(np.deg2rad(phi)), np.cos(np.deg2rad(phi))))*extra(t), axis=0)
+        x_obs += np.sum(np.vstack((np.sin(phi), np.cos(phi)))*extra(t), axis=0)
     # Measurement Error
     x_obs += np.random.normal(0, x_err)
 
@@ -356,3 +358,36 @@ def agis(r5d, t, phi, x_err, extra=None, epoch=2016.0, G=None):
     results['uwe']= np.sqrt(np.sum(R**2 / x_err**2)/(np.sum(weights>0.2)-nparam))
 
     return results
+
+# translates the results of a fit (should work with simple and gaia fit) to a params object
+def resultsparams(results,error=False,refra=np.NaN,refdec=np.NaN):
+    rparams=params()
+    efac=0
+    if error==True:
+        efac=1
+    if 'drac' in results.keys():
+        rparams.ra=results['ra_ref']
+        rparams.dec=results['dec_ref']
+        rparams.drac=results['drac'] + efac*np.random.randn()*results['drac_error']
+        rparams.ddec=results['ddec'] + efac*np.random.randn()*results['ddec_error']
+        rparams.pmrac=results['pmrac'] + efac*np.random.randn()*results['pmrac_error']
+    else:
+        if np.isnan(refra):
+            print("looks like you're using gaiafit results(?)")
+            print("in which case you need to supply a refra and refdec")
+            print("astromet likes to work in small (mas) differences, drac and ddec")
+            print("from a reference ra and dec")
+            print("(we'll use the ra and dec given for the moment)")
+            refra=results['ra']
+            refdec=results['dec']
+        rparams.ra=refra
+        rparams.dec=refdec
+        racoffset=(results['ra']-refra)*np.cos(refdec)/mas
+        decoffset=(results['dec']-refdec)/mas
+        rparams.drac=racoffset + efac*np.random.randn()*results['ra_error']
+        rparams.ddec=decoffset + efac*np.random.randn()*results['dec_error']
+        rparams.pmrac=results['pmra'] + efac*np.random.randn()*results['pmra_error']
+    rparams.pmdec=results['pmdec'] + efac*np.random.randn()*results['pmdec_error']
+    rparams.parallax=results['parallax'] + efac*np.random.randn()*results['parallax_error']
+
+    return rparams
