@@ -8,6 +8,7 @@ import sys
 import os
 from .lensing import *
 
+
 # Create units used elsewhere
 mSun = constants.M_sun.to(u.kg).value
 lSun = constants.L_sun.to(u.W).value
@@ -30,8 +31,6 @@ tbegin = 2014.6670  # time (in years) of Gaia's first observations
 # ----------------
 # -User functions
 # ----------------
-
-
 class params():
     def __init__(self):
         # astrometric parameters
@@ -85,7 +84,7 @@ def totalmass(ps):
 
 
 def Delta(ps):
-    ps.Delta = (ps.q-ps.l)/((1+ps.q)*(1+ps.l))
+    ps.Delta = (ps.l-ps.q)/((1+ps.q)*(1+ps.l))
     return ps.Delta
 
 
@@ -181,9 +180,8 @@ def design_matrix(ts, ra, dec, phis=None, epoch=2016.0):
 
     if np.size(phis) > 1:
         # sin and cos angles
-        angles = np.deg2rad(phis)
-        sina = np.sin(angles)
-        cosa = np.cos(angles)
+        sina = np.sin(phis)
+        cosa = np.cos(phis)
 
         # Construct design matrix
         design = design[0]*sina[:, None] + design[1]*cosa[:, None]
@@ -204,9 +202,9 @@ def barycentricPosition(time):
 
 
 # binary orbit
-def findEtas(ts, period, ecc, tPeri=0, N_it=10):
+def findEtas(ts, period, ecc, tPeri=0, N_it=10, precision=1e-5):
     # finds eccentric anomaly with iterative Halley's method
-    phase=2*np.pi*(((ts-tPeri)/period) % 1)
+    phase=2*np.pi*(((ts-tPeri)/period) ) # deleted a %1 from brackets (24/10/23) - hope nothing breaks!
 
     sph=np.sin(phase)
     cph=np.cos(phase)
@@ -215,7 +213,7 @@ def findEtas(ts, period, ecc, tPeri=0, N_it=10):
     deltaeta=1
 
     it=0
-    while ((np.max(np.abs(deltaeta))>1e-5) & (it<N_it)):
+    while ((np.max(np.abs(deltaeta))>precision) & (it<N_it)):
     # Halley's method
         it+=1
         sineta=np.sin(eta)
@@ -225,18 +223,61 @@ def findEtas(ts, period, ecc, tPeri=0, N_it=10):
         d2f= ecc*sineta
         deltaeta  = -f*df / (df*df - 0.5*f*d2f)
         eta      += deltaeta
-    # since the Halley method converges cubically, a correction < 1e-5 at the current iteration
+    # since the Halley method converges cubically, a correction < precision=1e-5 at the current iteration
     # implies that it would be <~1e-15 at the next iteration, which is beyond the precision limit
     return eta
 
+def findEtasHyperbolic(ts, tau, ecc, tPeri=0, N_it=10, precision=1e-5):
+    # finds eccentric anomaly for unbound orbits with modified iterative Halley's method
+    # initial guess
+    phase=2*np.pi*(ts-tPeri)/tau
+    zeta=np.arcsinh(phase/ecc)
+
+    deltazeta=1
+
+    it=0
+    while ((np.max(np.abs(deltazeta))>precision) & (it<N_it)):
+    # Halley's method
+        it+=1
+        sinhzeta=np.sinh(zeta)
+        coshzeta=np.cosh(zeta)
+        f  = ecc*sinhzeta - zeta - phase
+        df = ecc*coshzeta - 1.
+        d2f= ecc*sinhzeta
+        deltazeta  = -f*df / (df*df - 0.5*f*d2f)
+        zeta      += deltazeta
+    # since the Halley method converges cubically, a correction < precision=1e-5 at the current iteration
+    # implies that it would be <~1e-15 at the next iteration, which is beyond the precision limit
+    return zeta
+
+def findPhisParabolic(ts,tau,tPeri=0, N_it=10, precision=1e-5):
+    alpha=3*(ts-tPeri)/tau
+    phi=np.sign(alpha)*np.arccos((1-alpha**2)/(1+alpha**2))
+    deltaphi=1
+    it=0
+    while ((np.max(np.abs(deltaphi))>precision) & (it<N_it)):
+    # Halley's method
+        it+=1
+        sinphi=np.sin(phi)
+        cosphi=np.cos(phi)
+        f  = sinphi*(2+cosphi)/(1+cosphi)**2 - alpha
+        df = 3/(1+cosphi)**2
+        d2f= 6*sinphi/(1+cosphi)**3
+        deltaphi  = -f*df / (df*df - 0.5*f*d2f)
+        phi      += deltaphi
+    return phi
+
+
+
+
 
 def bodyPos(pxs, pys, l, q):  # given the displacements transform to c.o.m. frame
-    px1s = pxs*q/(1+q)
-    px2s = -pxs/(1+q)
-    py1s = pys*q/(1+q)
-    py2s = -pys/(1+q)
-    pxls = -pxs*(l-q)/((1+l)*(1+q))
-    pyls = -pys*(l-q)/((1+l)*(1+q))
+    px1s = -pxs*q/(1+q)
+    px2s = pxs/(1+q)
+    py1s = -pys*q/(1+q)
+    py2s = pys/(1+q)
+    pxls = pxs*(l-q)/((1+l)*(1+q))
+    pyls = pys*(l-q)/((1+l)*(1+q))
     return px1s, py1s, px2s, py2s, pxls, pyls
 
 
@@ -600,11 +641,11 @@ def radial_velocity(ts, ps, source='p'):
     if ps.a == 0:
         return np.zeros(ts.size)
     if (source == 'c') or (source == 'combined'):
-        Delta = (ps.q-ps.l)/((1+ps.q)*(1+ps.l))
+        Delta = (ps.l-ps.q)/((1+ps.q)*(1+ps.l))
     elif (source == 'p') or (source == 'primary'):
-        Delta = ps.q/(1+ps.q)
+        Delta = -ps.q/(1+ps.q)
     elif (source == 's') or (source == 'secondary'):
-        Delta = -1/(1+ps.q)
+        Delta = 1/(1+ps.q)
     etas = findEtas(ts, ps.period, ps.e, tPeri=ps.tperi)
     bracket = (np.cos(ps.vphi)*np.sin(etas) -
                np.sqrt(1-ps.e**2)*np.sin(ps.vphi)*np.cos(etas))/(1-ps.e*np.cos(etas))
@@ -649,7 +690,7 @@ def seperation(ts, ps, phis=None):
 # ----------------------
 # -Utilities
 # ----------------------
-# returns a number to a given significant digits (if extra true also returns exponent)
+
 
 def viewing_angles(la, i, ap):
     """Converts conentional orbital elements to our coordinate system
@@ -669,7 +710,7 @@ def viewing_angles(la, i, ap):
     vomega = np.arctan2(num, denom)
     return vtheta, vphi, vomega
 
-
+# returns a number to a given significant digits (if extra true also returns exponent)
 def sigString(number, significantFigures, extra=False):
     roundingFactor = significantFigures - int(np.floor(np.log10(np.abs(number)))) - 1
     rounded = np.round(number, roundingFactor)
@@ -683,8 +724,6 @@ def sigString(number, significantFigures, extra=False):
         return string, roundingFactor
 
 # generating, sampling and fitting a split normal (see https://authorea.com/users/107850/articles/371464-direct-parameter-finding-of-the-split-normal-distribution)
-
-
 def splitNormal(x, mu, sigma, cigma):
     epsilon = cigma/sigma
     alphas = sigma*np.ones_like(x)
@@ -729,3 +768,73 @@ def splitFit(xs):  # fits a split normal distribution to an array of data
     Z = ks[np.argmin(np.abs(phi_ks))]
 
     return xs[Z], sigma, cigma
+
+# converts a phase (phi in radians) to a time in years
+# rp should be in AU and Mtotal in Msun
+def phi_to_t(phi,rp,e,Mtotal,tPeri=0):
+    cphi=np.cos(phi)
+    sphi=np.sin(phi)
+    if e==1:
+        tau=np.sqrt((rp*(1+e))**3 /(Galt*Mtotal))
+        return tau*sphi*(2+cphi)/(3*(1+cphi)**2)+tPeri
+    if e<1:
+        norbit=np.floor((phi+np.pi)/(2*np.pi))
+        ceta=(cphi+e)/(1+e*cphi)
+        seta=np.sqrt(1-e**2)*sphi/(1+e*cphi)
+        eta=np.arctan2(seta,ceta)
+        tau=np.sqrt((rp/(1-e))**3 /(Galt*Mtotal))
+        return tau*(eta-e*seta+2*np.pi*norbit)+tPeri
+    if e>1:
+        chzeta=(cphi+e)/(1+e*cphi)
+        shzeta=np.sqrt(e**2 - 1)*sphi/(1+e*cphi)
+        zeta=np.arctanh(shzeta/chzeta)
+        tau=np.sqrt(np.abs(rp/(1-e))**3 /(Galt*Mtotal))
+        return tau*(e*shzeta-zeta)+tPeri
+
+# inverse of above function
+def t_to_phi(t,rp,e,Mtotal,tPeri=0):
+    if e==1:
+        tau=np.sqrt((rp*(1+e))**3 /(Galt*Mtotal))
+        phi=findPhisParabolic(t,tau,tPeri=tPeri)
+    if e<1:
+        tau=2*np.pi*np.sqrt((rp/(1-e))**3 /(Galt*Mtotal))
+        norbit=np.floor(0.5+(t-tPeri)/tau)
+        eta=findEtas(t,tau,e,tPeri=tPeri)
+        cphi=(np.cos(eta)-e)/(1-e*np.cos(eta))
+        sphi=np.sqrt(1-e**2)*np.sin(eta)/(1-e*np.cos(eta))
+        phi=np.arctan2(sphi,cphi)+2*np.pi*norbit
+    if e>1:
+        tau=2*np.pi*np.sqrt(np.abs(rp/(1-e))**3 /(Galt*Mtotal))
+        zeta=findEtasHyperbolic(t,tau,e,tPeri=tPeri)
+        cphi=(e-np.cosh(zeta))/(e*np.cosh(zeta)-1)
+        sphi=np.sqrt(e**2-1)*np.sinh(zeta)/(e*np.cosh(zeta)-1)
+        phi=np.arctan2(sphi,cphi)
+    return phi
+
+# ----------------------
+# -Plots
+# ----------------------
+def plottrack(ts,params,ax=0,s=5,c=None,alpha=0.5,lw=0,ls='-',nts=1000):
+    if ax==0:
+        ax=plt.gca()
+    if len(ts)==2: # if we just give two times treats these as start and end and generates nts times
+        ts=np.linspace(ts[0],ts[1],nts)
+    dracs,ddecs=track(ts, params)
+    if lw==0:
+        if c!=None:
+            ax.scatter(dracs,ddecs,c=c,s=s,alpha=alpha)
+        else:
+            ax.scatter(dracs,ddecs,s=s,alpha=alpha)
+    else:
+        if c!=None:
+            ax.plot(dracs,ddecs,c=c,alpha=alpha,lw=lw,ls=ls)
+        else:
+            ax.plot(dracs,ddecs,alpha=alpha,lw=lw,ls=ls)
+    return ax
+
+def plotresults(ts,results,error=False,refra=np.NaN,refdec=np.NaN,
+        ax=0,s=5,c=None,alpha=0.5,lw=0,ls='-',nts=1000):
+    from .fits import resultsparams
+    rparams=resultsparams(results,error=error,refra=refra,refdec=refdec)
+    ax=plottrack(ts,rparams,ax=ax,s=s,c=c,alpha=alpha,lw=lw,ls=ls,nts=nts)
+    return ax
